#ifndef _COLOR_DETECTOR_
#define _COLOR_DETECTOR_

#include "detector.h"
#include "cvblob.h"

/** Detects objects by detecting colors.
  * Blobs are detected by thresholding colors within a given range (input wide) around a
  * given color (input color).
  * The min and max colors of the detected colors range are shown on the lower canvas
  * of the interface.
  *
  * Inputs :
  * \li area : min max minimal in maximal blob sizes
  * \li color : R G B
  * \li wide : R G B (ranges)
  * \li saturation : level of saturation (default 0)
  * \li brightness : level of saturation (default 0)
  *
  * Hotkeys:
  *
  * \li 4 => R+ 1
  * \li r => R- 1
  * \li 5 => G+ 1
  * \li t => G- 1
  * \li 6 => B+ 1
  * \li y => B- 1
  *
  * \li 7 => Rrange+ 1
  * \li u => Rrange- 1
  * \li 8 => Grange+ 1
  * \li i => Grange- 1
  * \li 9 => Brange+ 1
  * \li o => Brange- 1
  *
  * \li b => change brightness mode
  * \li upkey   => Brightness+ 1
  * \li downkey => Brightness- 1
  *
  * \li s => change saturation mode
  * \li upkey   => Saturation+ 1
  * \li downkey => Saturation- 1
  *
  * \li p => Print color/range or brightness or saturation based on current mode
  *
  * \li e => Out of brightness/saturation mode
  *
  * @ingroup DETECTOR
  * @author Xavier Bouthillier */
class ColorDetector : public Tapir::Detector
{
  enum state_t {OFF,BRIGHTNESS,SATURATION};

public:
  ColorDetector ();
  ~ColorDetector () {};
  bool detect (IplImage** imgs, std::vector<TapirObject>& objects);
  bool init (const int width, const int height, const int imgs, unsigned int& add, const char* fname, const char* chapter);
  void deinit ();
  void notify (const long int key, const char* params);

protected:
  CvScalar get_min_color();
  CvScalar get_max_color();
  int saturate(int x);

  IplImage *_label, *_img, *_hsvframe, *_threshy, *_convert, *_converted;
  int _color[3], _wide[3] ,_area[2];
  int _offset;
  cvb::CvBlobs *_blobs;

  int _brightness,_saturation;

  std::vector<TapirObject> _lasts;
  state_t _state;

  /** Enhance colors via saturation and brightness */
  bool enhance_BGR_colors(IplImage* img);
};

#endif

