#include "FaceDetector.h"

FaceDetector::FaceDetector ()
{
}

bool FaceDetector::init (const int width, const int height, const int imgs, unsigned int& add, const char *fname, const char* chapter)
{
  ValueParser vp(fname, CHAPTER "Detector");
  char tmp[255];
  if (vp.get("cascade", tmp, 255) < 1) {
    EOUT("No cascade file specified.");
    return false;
  }

  vp.get("scaling", _scale, 1.5);
  vp.get("neighbors", _nb, 2);
  vp.get("min_size", _size, 60);

  vp.get("time", _time, false);
  
  _cascade = (CvHaarClassifierCascade*)cvLoad(tmp, 0, 0, 0);
  if (_cascade == 0) {
    EOUT("Cascade file \"" << tmp << "\" could not be opened.");
    return false;
  }

  _storage = cvCreateMemStorage(0);
  _gray  = cvCreateImage(cvSize(width,height), 8, 1);

  CvScalar colors[] = {
    {{0,0,255}},
    {{0,128,255}},
    {{0,255,255}},
    {{0,255,0}},
    {{255,128,0}},
    {{255,255,0}},
    {{255,0,0}},
    {{255,0,255}}
  };
  _colors = colors;

  _imgoffset = imgs;
  add = 1;
  return true;
}

void FaceDetector::deinit ()
{
  cvReleaseImage(&_gray);
}

bool FaceDetector::detect (IplImage** imgs, std::vector<TapirObject>& objects)
{
  IplImage *raw = imgs[0];
  if (raw == 0) return true;
  if (_gray == 0) return false;

  cvCvtColor(raw, _gray, CV_BGR2GRAY);
  cvEqualizeHist(_gray, _gray);
  cvClearMemStorage(_storage);

  double t = (double)cvGetTickCount();
  CvSeq* faces = cvHaarDetectObjects(_gray, _cascade, _storage, _scale, _nb, 0 | CV_HAAR_FIND_BIGGEST_OBJECT | CV_HAAR_DO_ROUGH_SEARCH | CV_HAAR_DO_CANNY_PRUNING | CV_HAAR_SCALE_IMAGE, cvSize(_size, _size));
  t = (double)cvGetTickCount() - t;
  double msec = t / ((double)cvGetTickFrequency()*1000.);
  if (_time)
    IOUT("Face detection time: " << msec << "ms");

  if (faces) {
    CvRect* r = (CvRect*)cvGetSeqElem(faces, 0);
    if (r) {
      _center.x = cvRound((r->x + r->width*0.5));
      _center.y = cvRound((r->y + r->height*0.5));
      cvCircle(raw, _center, 5, _colors[0], 3, 8, 0);
      objects.push_back(TapirObject(_center.x, _center.y, r->width*r->height));
    }
  }

  imgs[_imgoffset] = _gray;

  return true;
}

REGISTER(Detector, FaceDetector, "Detector that uses OpenCV face tracking.");

