#ifndef _OPENCV_FACE_DETECTOR_
#define _OPENCV_FACE_DETECTOR_

#include "detector.h"

using namespace Tapir;

/** Detector that uses OpenCV face detection.
  * The required config parameter \b cascade describes the path to
  * an OpenCV cascade description. Example files are included in
  * src/detector/FaceDetector/cascades.
  *
  * The optional boolean parameter \b time will cause the time
  * required for detection to be printed.
  *
  * The detection algorithm itself is parametrized as follows. In general, higher
  * values of the first two and lower values for the third result in faster
  * detection times, but at the cost of precision.
  * \li \b min_size defines the minimum height of a face that is to be detected.
  * \li \b scaling defines the factor by which the image should be resized
  *     in subsequent iterations of the detection step.
  * \li \b neighbors defines the mininimum number (minus 1) of neighbor rectangles
  *     that makes up an object.
  *
  * @ingroup DETECTOR
  * @author Thomas Lampe, Christian Mueller */
class FaceDetector : public Detector
{
 public:
  FaceDetector ();
  ~FaceDetector () {};
  bool detect (IplImage** imgs, std::vector<TapirObject>& objects);
  bool init (const int width, const int height, const int imgs, unsigned int& add, const char* fname, const char* chapter);
  void deinit ();

 protected:
  CvHaarClassifierCascade *_cascade;
  CvMemStorage *_storage;
  CvScalar *_colors;
  bool _time;
  IplImage *_gray;
  CvPoint _center;
  double _scale;
  int _size, _nb, _imgoffset;
};

#endif

