#include "ColoredGridDetector.h"

ColoredGridDetector::ColoredGridDetector ()
{
}

bool ColoredGridDetector::init (const int width, const int height, const int imgs, unsigned int& add, const char* fname, const char* chapter)
{
  // set where and how many images we generate
  _offset = imgs;
  add = 2;

  ValueParser vp(fname, CHAPTER "Detector");

  // grid size
  vp.get("grid_x",_grid_x, 10);
  vp.get("grid_y",_grid_y,_grid_x);

  vp.get("x_min",_x_min,0);
  vp.get("x_max",_x_max,height);
  vp.get("y_min",_y_min,0);
  vp.get("y_max",_y_max,width);

  int x_step = (_x_max-_x_min)/_grid_x;
  int y_step = (_y_max-_y_min)/_grid_y;
  if (x_step<=0 || y_step<=0) {
    EOUT("Grid graduation is too big for min-max values");
    return false;
  }

  _block_height = (_x_max - _x_min)/_grid_x;
  _block_width =  (_y_max - _y_min)/_grid_y;

  vp.get("print_block_color",_print_block_color,false);

  bool do_get_block_color;
  vp.get("get_block_color",do_get_block_color,false);
  vp.get("get_color_x",_get_color_x,0);
  vp.get("get_color_y",_get_color_y,0);
  if (do_get_block_color) _state = COLOR;
  else _state = OFF;

  vp.get("saturation",_saturation,0);
  vp.get("brightness",_brightness,0);

  // get the colors to detect
  char colors[255];

  vp.get("colors",colors,255,"0,0,0|255,255,255");

  std::string test(colors);

  int sindex = 0;
  int eindex = test.find('|');
  while (eindex != (int)std::string::npos) {
    _colors.push_back(read_color_str(test.substr(sindex,eindex)));
    sindex = eindex + 1;
    eindex = test.find('|',sindex);
  }

  _colors.push_back(read_color_str(test.substr(sindex)));

  _detect    = cvCreateImage(cvSize(width, height), IPL_DEPTH_8U, 3);
  _raw       = cvCreateImage(cvSize(width, height), IPL_DEPTH_8U, 3);
  _convert   = cvCreateImage(cvSize(width, height), IPL_DEPTH_8U, 3);
  _converted = cvCreateImage(cvSize(width, height), IPL_DEPTH_8U, 3);

  _channels = 3;

  _blocks = (int*)malloc(2*_grid_x*_grid_y*_channels*sizeof(int));
  if ( _blocks == NULL ) {
    EOUT("Failed to allocate memory for the grid");
    return false;
  }

  for (int x=0; x<_grid_x; x++) {
    for (int y=0; y<_grid_y; y++) {
      for (int k=0; k<_channels; k++) {
        _blocks[get_block_index(x,y,k)] = 0;
      }
    }
  }

  ///< \todo saturation and brightness do not belong here
  REGISTER_KEY(ColoredGridDetector, 'l', "select parameter: lower bounds");
  REGISTER_KEY(ColoredGridDetector, 'u', "select parameter: upper bounds");
  REGISTER_KEY(ColoredGridDetector, 'b', "select parameter: brightness");
  REGISTER_KEY(ColoredGridDetector, 's', "select parameter: saturation");
  REGISTER_KEY(ColoredGridDetector, 'c', "select parameter: get color from a block");
  REGISTER_KEY(ColoredGridDetector, 'e', "OFF");
  REGISTER_KEY(ColoredGridDetector, 'p', "print currently selected parameter");
  REGISTER_EXTENDED_KEY(ColoredGridDetector, "ARROW_UP",    27,91,65, "increase currently selected parameter (x)");
  REGISTER_EXTENDED_KEY(ColoredGridDetector, "ARROW_DOWN",  27,91,66, "decrease currently selected parameter (x)");
  REGISTER_EXTENDED_KEY(ColoredGridDetector, "ARROW_RIGHT", 27,91,67, "increase currently selected parameter (y)");
  REGISTER_EXTENDED_KEY(ColoredGridDetector, "ARROW_LEFT",  27,91,68, "decrease currently selected parameter (y)");

  return true;
}

void ColoredGridDetector::notify (const long int key, const char* params)
{
  switch (key) {
  case 'l' :
    IOUT(" Change lower bounds mode");
    _state = LOWER;
    break;

  case 'u' :
    IOUT(" Change upper bounds mode");
    _state = UPPER;
    break;

  case 'b' :
    IOUT(" Change brightness mode");
    _state = BRIGHTNESS;
    break;

  case 's' :
    IOUT(" Change saturation mode");
    _state = SATURATION;
    break;

  case 'c' :
    IOUT(" Get color from a block mode");
    _state = COLOR;
    break;

  case 'e' :
    IOUT(" OFF");
    _state = OFF;
    break;

  case 'p' :
    switch (_state) {
    case LOWER :
    case UPPER :
      IOUT(" x_min = " << _x_min);
      IOUT(" x_max = " << _x_max);
      IOUT(" y_min = " << _y_min);
      IOUT(" y_max = " << _y_max);
      break;

    case COLOR :
      IOUT(key << " :: get_color for block ( " << _get_color_x << ", " << _get_color_y << ")");
      get_block_color();
      break;

    case BRIGHTNESS :
      IOUT(" brightness = " << _brightness);
      break;

    case SATURATION :
      IOUT(" saturation = " << _saturation);
      break;

    default :
      break;
    }
    break;

  case 4283163 : // UP key
    switch (_state) {
    case LOWER :
      if ( _x_max > 2 ) _x_max -= 2;
      else _x_max = 0;
      break;

    case UPPER :
      if ( _x_min > 2 ) _x_min -= 2;
      else _x_min = 0;
      break;

    case COLOR :
      _get_color_x = (_grid_x+_get_color_x-1)%_grid_x;
      break;

    case BRIGHTNESS :
      _brightness += 5;
      break;

    case SATURATION :
      _saturation += 2;
      break;
    case OFF :
      break;
    }
    break;

  case 4348699 : // DOWN key
    switch (_state) {
    case LOWER :
      _x_max += 2;
      break;

    case UPPER :
      _x_min += 2;
      break;

    case COLOR :
      _get_color_x = (_get_color_x + 1)%_grid_x;
      break;

    case BRIGHTNESS :
      _brightness -= 5;
      break;

    case SATURATION :
      _saturation -= 2;
      break;
    case OFF :
      break;
    }
    break;

  case 4414235 : // RIGHT key
    switch (_state) {
    case LOWER :
      _y_min += 2;
      break;

    case UPPER :
      _y_max += 2;
      break;

    case COLOR :
      _get_color_y = (_get_color_y + 1)%_grid_y;
      break;

    case BRIGHTNESS :
      _brightness += 5;
      break;

    case SATURATION :
      _saturation += 2;
      break;
    case OFF :
      break;
    }
    break;

  case 4479771 : // LEFT key
    switch (_state) {
    case LOWER :
      if ( _y_min > 2 ) _y_min -= 2;
      else _y_min = 0;
      break;

    case UPPER :
      if ( _y_max > 2 ) _y_max -= 2;
      else _y_max = 0;
      break;

    case COLOR :
      _get_color_y = (_grid_y+_get_color_y-1)%_grid_y;
      break;

    case BRIGHTNESS :
      _brightness -= 5;
      break;

    case SATURATION :
      _saturation -= 2;
      break;
    case OFF :
      break;
    }

    break;
  }
  return;
}

bool ColoredGridDetector::detect (IplImage** imgs, std::vector<TapirObject>& objects)
{
  if (imgs[0] == 0) return true;
  cvCopyImage(imgs[0], _raw);

  // get the image data
  uchar *data;
  data = (uchar *)_raw->imageData;

  // set values to 0
  for (int x=0; x<_grid_x; x++) {
    for (int y=0; y<_grid_y; y++) {
      for (int k=0; k<_channels; k++) {
        _blocks[get_block_index(x,y,k)] = 0;
      }
    }
  }

  int h,j;

  // saturation and brightness
  enhance_BGR_colors(_raw);

  // reset _detect to 0,0,0
  cvSet(_detect,CV_RGB(0,0,0));

  // calculate histogram
  get_grid(_raw);

  // create objects list
  objects.clear();

  for (h=0; h<_grid_x; h++) {
    for (j=0; j<_grid_y; j++) {

      if ( _print_block_color ) {
        cvRectangle(_raw,
                    cvPoint(j*_block_width+_y_min+5,h*_block_height+_x_min+5),
                    cvPoint((j+1)*_block_width+_y_min-5,(h+1)*_block_height+_x_min-5),
                    CV_RGB(get_block_value(h,j,2),get_block_value(h,j,1),
                           get_block_value(h,j,0)),3);
      }

      int closest = 255*255+255*255+255*255; // worst case
      int color_index = 0;
      int dist = 0;

      // get closest color
      for ( int c=0; c<(int)_colors.size(); c++ ) {
        dist = distance_to_color(get_block_value(h,j,0),
                                 get_block_value(h,j,1),
                                 get_block_value(h,j,2),
                                 _colors[c].val[0],_colors[c].val[1],_colors[c].val[2]);
        if ( closest > dist ) {
          closest = dist;
          color_index = c;
        }
      }

      // fill rectangle with closest color detected (bottom image)
      cvRectangle(_detect,
                  cvPoint(j*_block_width+_y_min+2,h*_block_height+_x_min+2),
                  cvPoint((j+1)*_block_width+_y_min-2,(h+1)*_block_height+_x_min-2),
                  _colors[color_index],CV_FILLED);

      objects.push_back(TapirObject(j*_block_width+_y_min,h*_block_height+_x_min,color_index));
    }
  }

  // generate output image
  draw_grid(_raw,    cvScalar(100,100,180));
  draw_grid(_detect, cvScalar(255,255,255));

  // to see the color of a given square
  draw_get_block_color();

  imgs[_offset]   = _raw;
  imgs[_offset+1] = _detect;

  return true;
}

int ColoredGridDetector::distance_to_color(int R1, int G1, int B1, int R2, int G2, int B2)
{
  return (R1-R2)*(R1-R2)+(G1-G2)*(G1-G2)+(B1-B2)*(B1-B2);
}

bool ColoredGridDetector::draw_get_block_color()
{
  if ( _state == COLOR ) {
    int _block_width =  (_y_max - _y_min)/_grid_y;

    cvRectangle(_detect, cvPoint(_y_min+_block_width*_get_color_y+2,_x_min+_block_height*_get_color_x+2),
                cvPoint(_y_min+_block_width*(_get_color_y+1)-2,_x_min+_block_height*(_get_color_x+1)-2),
                cvScalar(get_block_value(_get_color_x,_get_color_y,0), get_block_value(_get_color_x,_get_color_y,1), get_block_value(_get_color_x,_get_color_y,2)),6);
    return true;
  }
  return false;
}


bool ColoredGridDetector::get_block_color()
{
  for ( int i=0; i<(int)_colors.size(); i++ ) {
    int dist = distance_to_color(get_block_value(_get_color_x,_get_color_y,0),
                                 get_block_value(_get_color_x,_get_color_y,1),
                                 get_block_value(_get_color_x,_get_color_y,2),
                                 _colors[i].val[0],_colors[i].val[1],_colors[i].val[2]);

    IOUT("color : " << i << " : ( " << _colors[i].val[0] << ", ");
    IOUT( _colors[i].val[1] << ", ");
    IOUT(_colors[i].val[2] << ") : ");
    IOUT("( " << get_block_value(_get_color_x,_get_color_y,0) << ", ");
    IOUT(get_block_value(_get_color_x,_get_color_y,1) << ", ");
    IOUT(get_block_value(_get_color_x,_get_color_y,2) << ") : ");
    IOUT(dist << "\n");
  }

  return true;
}

bool ColoredGridDetector::draw_grid(IplImage* img, CvScalar color)
{
  // render grid
  for (int x=_x_min; x<=_x_max; x+=_block_height) {
    cvLine(img,cvPoint(_y_min,x),
           cvPoint(_y_max,x),color,1);
  }
  for (int y=_y_min; y<=_y_max; y+=_block_width) {
    cvLine(img,cvPoint(y,_x_min),
           cvPoint(y,_x_max),color,1);
  }

  return true;
}

bool ColoredGridDetector::enhance_BGR_colors(IplImage* img)
{
  if (_saturation!=0 || _brightness!=0) {
    cvCvtColor(img, _convert, CV_BGR2HSV);

    int height    = img->height;
    int width     = img->width;
    int step      = img->widthStep;
    int channels  = img->nChannels;
    uchar* data = (uchar *)_convert->imageData;

    for (int h=0; h<height; h++) {
      for (int j=0; j<width; j++) {
        data[h*step+j*channels+1] = saturate(data[h*step+j*channels+1]+_saturation);
        data[h*step+j*channels+2] = saturate(data[h*step+j*channels+2]+_brightness);
      }
    }

    cvCvtColor(_convert, _converted, CV_HSV2BGR);

    uchar* imgdata = (uchar *)img->imageData;
    uchar* convdata = (uchar *)_converted->imageData;
    for (int h=0; h<height; h++) {
      for (int j=0; j<width; j++) {
        for (int k=0; k<channels; k++)
          imgdata[h*step+j*channels+k] = saturate(convdata[h*step+j*channels+k]);
      }
    }

    return true;
  }

  return false;
}

bool ColoredGridDetector::get_grid (IplImage* img)
{
  int height    = img->height;
  int width     = img->width;
  int step      = img->widthStep;
  int channels  = img->nChannels;
  uchar *data;
  data      = (uchar *)img->imageData;

  height = _detect->height;
  width  = _detect->width;
  int x_step = (_x_max-_x_min)/_grid_x;
  int y_step = (_y_max-_y_min)/_grid_y;

  for (int x=0; x<height; x++) {
    for (int y=0; y<width; y++) {

      if ( x < _x_min || x > _x_max || y < _y_min || y > _y_max ) {
        data[x*step+y*channels+0] = saturate(data[x*step+y*channels+0]*0.5-50);
        data[x*step+y*channels+1] = saturate(data[x*step+y*channels+1]*0.5-50);
        data[x*step+y*channels+2] = saturate(data[x*step+y*channels+2]*0.5-50);
      } else {
        for (int k=0; k<channels; k++)
          _blocks[get_block_index((x-_x_min)/x_step,(y-_y_min)/y_step,k)] += data[x*step+y*channels+k];
      }
    }
  }

  return true;
}

int ColoredGridDetector::get_block_index (int x, int y, int z)
{
  return x*_grid_y*_channels + y*_channels + z;
}

int ColoredGridDetector::get_block_value(int x, int y, int z)
{
  return _blocks[get_block_index(x,y,z)]/(_block_height*_block_width);
}

CvScalar ColoredGridDetector::read_color_str(std::string colors)
{
  std::string integer;

  int RGB[3];
  int color = 0;
  for (int i=0; i<(int)colors.size(); i++ ) {
    if ( colors[i]==',' ) {
      RGB[color] = atoi(integer.c_str());
      color++;
      integer = "";
    } else if ( colors[i]=='|' ) {
      RGB[color] = atoi(integer.c_str());
      return cvScalar(RGB[0],RGB[1],RGB[2]);
    } else {
      integer = integer + colors[i];
    }
  }

  RGB[color] = atoi(integer.c_str());

  return cvScalar(RGB[0],RGB[1],RGB[2]);
}

int ColoredGridDetector::saturate (int value)
{
  if ( value > 255 ) return 255;
  if ( value < 0 ) return 0;
  return value;
}

void ColoredGridDetector::deinit ()
{
  free(_blocks);
  cvReleaseImage(&_detect);
  cvReleaseImage(&_raw);
  cvReleaseImage(&_convert);
  cvReleaseImage(&_converted);
}

ColoredGridDetector::~ColoredGridDetector ()
{
  deinit();
}

REGISTER(Detector, ColoredGridDetector, "Detect closest color of every blocks in a grid");
