#include "MazeDetector.h"

MazeDetector::MazeDetector ()
{
}

bool MazeDetector::init (const int width, const int height, const int imgs, unsigned int& add, const char* fname, const char* chapter) 
{
  if(! ColoredGridDetector::init( width, height, imgs, add, fname, chapter))
    return false;

  ValueParser vp(fname, CHAPTER "Detector");
 
  char ground[255];
  char walls[255];

  vp.get("ground", ground, 255, "255,255,255");
  vp.get("walls",  walls,  255, "0,0,0");

  _ground_color = ColoredGridDetector::read_color_str(std::string(ground));
  _wall_color   = ColoredGridDetector::read_color_str(std::string(walls));

  vp.get("max_distance_to_color",_max_dist_color, 150);

  return true;
}

void MazeDetector::deinit ()
{
  ColoredGridDetector::deinit();
}

bool MazeDetector::detect (IplImage** imgs, std::vector<TapirObject>& objects)
{
  if (imgs[0] == 0) return true;
  cvCopy(imgs[0], _raw);

  // get the image data
  uchar *data;
  data = (uchar *)_raw->imageData;

  // set values to 0
  for(int x=0;x<_grid_x;x++){
    for(int y=0;y<_grid_y;y++){
      for(int k=0;k<_channels;k++){
        _blocks[ColoredGridDetector::get_block_index(x,y,k)] = 0;
      }
    }
  }

  int h,j;

  // saturation and brightness
  ColoredGridDetector::enhance_BGR_colors(_raw);

  // reset _detect to 0,0,0
  cvSet(_detect,CV_RGB(0,0,0));

  // calculate histogram
  ColoredGridDetector::get_grid(_raw);

  int grid_height = (_x_max - _x_min)/_grid_x;
  int grid_width =  (_y_max - _y_min)/_grid_y;

  // create objects list
  objects.clear();

  for(h=0;h<_grid_x;h++){
    for(j=0;j<_grid_y;j++){
      if( _print_block_color ){
        cvRectangle(_raw,
                    cvPoint(j*grid_width+_y_min+5,h*grid_height+_x_min+5),
                    cvPoint((j+1)*grid_width+_y_min-5,(h+1)*grid_height+_x_min-5),
                           CV_RGB(ColoredGridDetector::get_block_value(h,j,2),
                                  ColoredGridDetector::get_block_value(h,j,1),
                                  ColoredGridDetector::get_block_value(h,j,0)),3);
      }

      int closest = 255*255+255*255+255*255; // worst case
      int color_index = 0;
      int dist = 0;

      // get closest color
      for( int c=0; c<(int)_colors.size(); c++ ){
        dist = ColoredGridDetector::distance_to_color(ColoredGridDetector::get_block_value(h,j,0),
                                 ColoredGridDetector::get_block_value(h,j,1),
                                 ColoredGridDetector::get_block_value(h,j,2),
                                 _colors[c].val[0],_colors[c].val[1],_colors[c].val[2]);
        if( closest > dist ){
          closest = dist;
          color_index = c;
        }
      }

      int dist_to_ground_color = ColoredGridDetector::distance_to_color(
                         ColoredGridDetector::get_block_value(h,j,0),
                         ColoredGridDetector::get_block_value(h,j,1),
                         ColoredGridDetector::get_block_value(h,j,2),
                         _ground_color.val[0],_ground_color.val[1],_ground_color.val[2]);


      int dist_to_wall_color = ColoredGridDetector::distance_to_color(
                         ColoredGridDetector::get_block_value(h,j,0),
                         ColoredGridDetector::get_block_value(h,j,1),
                         ColoredGridDetector::get_block_value(h,j,2),
                         _wall_color.val[0],_wall_color.val[1],_wall_color.val[2]);

      // closest color is a given color
      if( closest <= _max_dist_color ){

        cvRectangle(_detect,
                    cvPoint(j*grid_width+_y_min+2,h*grid_height+_x_min+2),
                    cvPoint((j+1)*grid_width+_y_min-2,(h+1)*grid_height+_x_min-2),
                            _colors[color_index],CV_FILLED);

        objects.push_back(TapirObject(h*grid_height+_x_min,j*grid_width+_y_min,color_index+2));
      // closest color is the ground color
      }else if( dist_to_ground_color < dist_to_wall_color ){
        cvRectangle(_detect,
                    cvPoint(j*grid_width+_y_min+2,h*grid_height+_x_min+2),
                    cvPoint((j+1)*grid_width+_y_min-2,(h+1)*grid_height+_x_min-2),
                            _ground_color,CV_FILLED);

        objects.push_back(TapirObject(h*grid_height+_x_min,j*grid_width+_y_min,0));
      // closest color is the wall color
      }else{
        cvRectangle(_detect,
                    cvPoint(j*grid_width+_y_min+2,h*grid_height+_x_min+2),
                    cvPoint((j+1)*grid_width+_y_min-2,(h+1)*grid_height+_x_min-2),
                            _wall_color,CV_FILLED);

        objects.push_back(TapirObject(h*grid_height+_x_min,j*grid_width+_y_min,1));
      } 
    }
  }


  // generate output image
  ColoredGridDetector::draw_grid(_detect,cvScalar(255,255,255));
  ColoredGridDetector::draw_grid(_raw,cvScalar(100,100,180));

  // to see the color of a given square
  ColoredGridDetector::draw_get_block_color();

  imgs[_offset]   = _raw;
  imgs[_offset+1] = _detect;

  return true;
}

REGISTER(Detector, MazeDetector, "Detect a maze as a grid");
