#ifndef _HISTOGRAM_DETECTOR_
#define _HISTOGRAM_DETECTOR_

#include "detector.h"
#include "HistogramFeature.h"
#include "cvblob.h"

using namespace Tapir;

/** Detects objects by comparing histograms.
  * During the first few frames after program start, the detector will accumulate
  * the colors contained in the image, and generate a corresponding histogram.
  * Afterwards, any new frame will be compared against this background histogram,
  * and any areas of colors not contained therein will be detected as objects.
  *
  * Additional background examples can be added by either pressing \b + once,
  * which will add a single frame; or by pressing \b space, which will cause
  * the background histogram to be updated until space is pressed a second time.
  *
  * The number of frames to be added after start is 50 by default, and can be set
  * through the config parameter \b examples_init. The number of frames to be
  * added after pressing + can be increased likewise by setting \b examples_renew.
  *
  * The boolean parameter \box can be set to false to suppress the drawing of
  * boxes around all detected object in the source image.
  *
  * The parameter \area, which consists of two floating-point values, defines the
  * minimum and maximum size that a foreground color region must be within in
  * order to be considered an object.
  *
  * The current histograms can be saved by pressing the key \b s, with the file name
  * being defined by the parameter \b filename, or \e histograms.xml by default.
  * Saved histograms can be loaded by pressing \b l or by setting the parameter
  * \b autoload to \e true.
  *
  * @ingroup DETECTOR
  * @author Thomas Lampe, Stefan Welker */
class HistogramDetector : public Detector
{
 public:
  HistogramDetector ();
  ~HistogramDetector () {};
  bool detect (IplImage** imgs, std::vector<TapirObject>& objects);
  bool init (const int width, const int height, const int imgs, unsigned int& add, const char* fname, const char* chapter);
  void deinit ();
  void notify (const long int cmd, const char* params);

 protected:
  HistogramFeature::Histogram *_neg, *_pos, *_dif;
  IplImage *_detect, *_label, *_canvas;
  int _updates, _ex[2], _area[2], _imgoffset;
  bool _first, _init, _lock, _autoload;
  cvb::CvBlobs *_blobs;
  char *_filename;
  bool load ();
};

#endif

