#include "HistogramFeature.h"

namespace HistogramFeature {

Histogram::Histogram ()
{
  rbins = 16;
  gbins = 16;
  bbins = 16;
  histSize[0] = rbins;
  histSize[1] = gbins;
  histSize[2] = bbins;
  rranges[0] = 0;
  rranges[1] = 255;
  granges[0] = 0;
  granges[1] = 255;
  branges[0] = 0;
  branges[1] = 255;
  ranges[0] = rranges;
  ranges[1] = granges;
  ranges[2] = branges;
  once = true;
  scale = 4;
  accumulate = 0;
  hist = cvCreateHist(3,histSize,CV_HIST_ARRAY,ranges,1);
  hist_img_r_g_b = cvCreateImage(cvSize(rbins*scale+bbins*scale,gbins*scale),8,3);
}

void Histogram::process (IplImage* image)
{
  if (once) {
    hist_img_r = cvCreateImage(cvGetSize(image),8,1);
    hist_img_g = cvCreateImage(cvGetSize(image),8,1);
    hist_img_b = cvCreateImage(cvGetSize(image),8,1);
    planes[0] = hist_img_r;
    planes[1] = hist_img_g;
    planes[2] = hist_img_b;
    once = false;
  }
  cvSplit( image, hist_img_r, hist_img_g, hist_img_b, 0 );
  cvCalcHist(planes,hist,accumulate,0);
}

IplImage* Histogram::display ()
{
  float maxVal=0;
  cvGetMinMaxHistValue(hist,0,&maxVal,0,0);

  for (int b=0; b<bbins; b++) {
    for (int r=0; r<rbins; r++) {
      for (int g=0; g<gbins; g++) {
        float binVal = (float)cvGetReal3D(hist->bins, r, g, b);
        int intensity = cvRound(binVal*255/maxVal);
        intensity = 0;
        if (binVal > 0) intensity = 1;
        cvRectangle(hist_img_r_g_b,cvPoint(r*scale+b*scale,g*scale),cvPoint((r+1)*scale-1+b*scale,(g+1)*scale-1),CV_RGB(intensity*r,intensity*g,intensity*b),CV_FILLED);
      }
    }
  }
  return hist_img_r_g_b;
}

void Histogram::normalize (float n)
{
  cvNormalizeHist(hist,n);
}

void Histogram::clear ()
{
  if (!once) cvClearHist(hist);
}

void Histogram::subtract_histogram (CvHistogram* h1, CvHistogram* h2, CvHistogram* dst)
{
  // NEED TO BE OF SAME TYPE!!
  if (!CV_IS_HIST(h1) || !CV_IS_HIST(h2) || !CV_IS_HIST(dst))
    EOUT("Invalid histogram pointer[s]");

  CvArr* arrs[] = { h1->bins, h2->bins, dst->bins };
  CvMatND stubs[3];
  CvNArrayIterator iterator;
  cvInitNArrayIterator( 3, arrs, 0, stubs, &iterator );
  if ( CV_MAT_TYPE(iterator.hdr[0]->type) != CV_32FC1 )
    EOUT("All histograms must have 32fC1 type");

  do {
    const float* data1 = (const float*)(iterator.ptr[0]);
    const float* data2 = (const float*)(iterator.ptr[1]);
    float* dstdata = (float*)(iterator.ptr[2]);
    int i;
    for ( i = 0; i < iterator.size.width; i++ ) {
      float pos = data1[i];
      float neg = data2[i];
      dstdata[i]=255*(pos-neg)/pos;
      if (dstdata[i]<0)dstdata[i]=0;
    }
  } while ( cvNextNArraySlice( &iterator ));
}

}

