#ifndef _MULTI_OBJECT_TLD_DETECTOR_
#define _MULTI_OBJECT_TLD_DETECTOR_

#include "detector.h"
#include "motld/MultiObjectTLD.h"

using namespace Tapir;

/** Detects objects using a multi-object color variant of the TLD
  * algorithm (a.k.a. Predator) by Zdenek Kalal.
  * Adapted from the motld project by Christian Lutz and Thorsten
  * Engesser (https://github.com/evilsantabot/motld).
  *
  * Available parameters:
  * \li \b learning (bool, \e true): if set to \e false, the classifier
  *     will not be updated; initial setting, may be changed through key
  *     events later
  * \li \b draw_mode (int, 255): unknown
  * \li \b classifier (string, \e classifier.tld): path to classifier
  *     file for saving and loading
  * \li \b autoload (bool, \e false) if set to \e true, the specified
  *     classifier file will be loaded at startup
  * \li \b color (bool, \e true): whether to use color information
  *     for distinction between multiple objects or not
  *
  * Available hotkeys:
  * \li \b d: toggles drawing of detections
  * \li \b t: toggles drawing of crosses
  * \li \b p: toggles drawing of patches
  * \li \b l: toggles update of classifiers
  * \li \b r: resets classifiers
  * \li \b s: saves classifiers to file
  * \li \b o: loads classifiers from file
  * \li \b L: (internal use) alternatingly sets the start and
  *     end corners of an object frame; expects two integer values
  *     as parameters
  * \li \b R: (internal use) resets a start corner
  *
  * @ingroup DETECTOR
  * @author Christian Lutz and Thorsten Engesser (TLD), Thomas Lampe (module) */
class MOTLDDetector : public Detector
{
 public:
  MOTLDDetector ();
  ~MOTLDDetector () {};
  bool detect (IplImage** imgs, std::vector<TapirObject>& objects);
  bool init (const int width, const int height, const int imgs, unsigned int& add, const char* fname, const char* chapter);
  void deinit ();
  void notify (const long int cmd, const char* params);

 protected:
  IplImage *_canvas;
  MultiObjectTLD *_tld;
  int _i, _width, _height, _size, _imgoffset, _drawmode, _coord;
  bool _learning, _save, _load, _reset, _corner;
  unsigned char *_data;
  char *_classfile, _buf[MAX_STR_LEN], *_token;
  Matrix _red, _green, _blue;
};

#endif

