#include "tcpserver.h"
#include <cstdlib>
#include <sys/time.h>
#include <sys/param.h>

#ifdef __BYTE_ORDER
  # if __BYTE_ORDER != __LITTLE_ENDIAN
    Error: no support for big-endian systems yet!
  #endif
#endif

#define SSAPPEND(xxx) { ss << #xxx << ":" << ecvt(objects[_i].xxx, 8, &_tmpi, &_tmpi) << ";"; };

bool TCPServer::init (const int width, const int height, const int imgs, unsigned int& add, const char* fname, const char* chapter)
{
  ValueParser vp(fname, CHAPTER "Output");

  vp.get("port", _port, 7102);
  _server = new TCPServerSocket(_port, 1);
  _socket = _server->accept();
  std::cout << "Connected to client on port " << _port << "." << std::endl;
  gettimeofday(&_start, NULL);

  return true;
}

bool TCPServer::send (std::vector<TapirObject>& objects, const timeval& time)
{
  try {
    _socket->recv((void*)_in, 2);
    if (_in[0] != '#' || _in[1] != '#') {
      EOUT("Faulty ping!");
      return false;
    }

    //_runtime = 1000. * (time.tv_sec-_start.tv_sec) + (time.tv_usec-_start.tv_usec) / 1000.;
    _out[0] = _out[1] = '*';
    memcpy((void*)&_out[ 2], (void*)&time, 8);
    _send = 10;

    if (objects.size() > 0) {
      memcpy((void*)&_out[10], (void*)&objects[0].x, 8);
      memcpy((void*)&_out[18], (void*)&objects[0].y, 8);
      memcpy((void*)&_out[26], (void*)&objects[0].size, 8);
      memcpy((void*)&_out[34], (void*)&objects[0].angle, 8);
      _send += 32;
    }

    _socket->send((void*)_out, _send);
  } catch (SocketException e) {
    EOUT("Sending failed: " << e.what());
    return false;
  }
  return true;
}

REGISTER(Output, TCPServer, "Sends a single object via TCP/IP.");

