#ifndef _TAPIR_TCP_SERVER_H_
#define _TAPIR_TCP_SERVER_H_

#include "output.h"
#include "Socket.h"
#include <sstream>

using namespace Tapir;

/** Allows clients to receive object information via TCP/IP.
  * Clients must send the signal '##' to request object information.
  * The response consists of the start sequence '**', followed by the
  * image acquisition time, as well as the x and y coordinates, size
  * and angle of each object. Each numeric value is a little-endian
  * 64-bit floating-point number.
  *
  * The TCP port is defined by the config parameter \b port.
  *
  * @ingroup OUTPUT
  * @author Thomas Lampe */
class TCPServer : public Output
{
 public:
  bool init (const int width, const int height, const int imgs, unsigned int& add, const char* fname, const char* chapter);
  bool send (std::vector<TapirObject>& objects, const timeval& time);

 protected:
  TCPSocket *_socket;
  TCPServerSocket *_server;
  int _port, _tmpi;
  char _out[255];
  unsigned char _in[128];
  std::stringstream ss;
  unsigned int _i, _send;
  timeval _start;
  double _runtime;
};

#endif

