#ifndef _NEURO_PREDICTOR_H_
#define _NEURO_PREDICTOR_H_

#include "tracker.h"
#include "n++.h"
#include "PatternSet.h"

using namespace Tapir;

/** Feeds the objects' coordinates to a neural network and visualizes
  * the net output. Intended to accompany the CLSquare module of the
  * same name.
  *
  * The net is loaded from the location specified by \b filename and
  * required to have an input dimensionality of 2 and an output of 1.
  *
  * The module generates one image, which may be customized in a
  * number of ways.
  * \li The \b background parameter can be set to either \e prediction,
  * in which case a smooth map of the output value will be generated,
  * or to \e binary, which will plot the two classes of input regions
  * lying above and below a specified \b threshold. The resolution of
  * the graph is controlled by the parameter \b map_step, which is 0.1
  * by default. If the background is set to \e camera or not specified,
  * the camera image will be used.
  * \li The parameter \b patterns may be set to the location of a valid
  * n++ pattern set, which will be drawn over the backgorund. \b Note:
  * using this option may cause instability and should be used with
  * caution.
  * \li The output of the network for visible objects will be plotted
  * as binary by default, with ones below the \b threshold marked in
  * green and those above in red. If the parameter \b binary_cursor
  * is set to \e false, a color gradient corresponding to the net
  * output will be used.
  *
  * Lastly, by pressing the key \b r, the network and pattern are
  * reloaded and the background plotted anew.
  *
  * @ingroup TRACKER
  * @author Thomas Lampe */
class NeuroPredictor : public Tracker
{
 public:
  bool init (const int width, const int height, const int imgs, unsigned int& add, const char* fname, const char* chapter);
  void select (IplImage** imgs, std::vector<TapirObject>& objects);
  void notify (const long int cmd, const char* params);
  inline void deinit ();

 protected:
  bool reload ();
  Net _net;
  int _i, _imgid, _width, _height;
  IplImage *_canvas, *_map;
  double _threshold, _step;
  bool _use_map, _bin_map, _bin_cursor, _use_patterns;
  char *_filename, *_patfile;
  PatternSet _patterns;
};

#endif
