#include "TriangleTracker.h"

TriangleTracker::TriangleTracker ()
{
}

bool TriangleTracker::init (const int width, const int height, const int imgs, unsigned int& add, const char* fname, const char* chapter)
{
  _offset = imgs;
  add = 1;

  _img = cvCreateImage(cvSize(width, height), IPL_DEPTH_8U, 3);
  return true;
}

void TriangleTracker::deinit ()
{
  cvReleaseImage(&_img);
}

void TriangleTracker::select (IplImage** imgs, std::vector<TapirObject>& objects)
{
  IplImage* raw = imgs[0];
  if (raw == 0) return;

  cvCopy(raw, _img);
  imgs[_offset] = _img;

  if (objects.size() < 3) return;

  // Get the three points
  std::vector<TapirObject> three_points;

  // Get three blobs with highest size
  for (int i=0; i<(int)objects.size(); i++)
    insert_if_bigger(three_points, objects[i]);

  TapirObject p1;
  TapirObject p2;

  if (three_points.size() != 3) {
    EOUT("Did not manage to extract points");
    return;
  }

  int qdist01 = point_qdist(three_points[0], three_points[1]);
  int qdist12 = point_qdist(three_points[1], three_points[2]);
  int qdist02 = point_qdist(three_points[0], three_points[2]);

  // merge two points that are closer
  if (qdist01 <= qdist12 && qdist01 <= qdist02) {
    p1 = three_points[2];
    p2 = TapirObject((three_points[0].x+three_points[1].x)/2,(three_points[0].y+three_points[1].y)/2,three_points[0].size+three_points[1].size);
  } else if (qdist12 <= qdist02 && qdist12 <= qdist01) {
    p1 = three_points[0];
    p2 = TapirObject((three_points[1].x+three_points[2].x)/2,(three_points[1].y+three_points[2].y)/2,three_points[1].size+three_points[2].size);
  } else {
    p1 = three_points[1];
    p2 = TapirObject((three_points[0].x+three_points[2].x)/2,(three_points[0].y+three_points[2].y)/2,three_points[0].size+three_points[2].size);
  }

  double angle = calculate_angle(p1,p2);
  objects.clear();
  objects.push_back(TapirObject((p1.x+p2.x)/2,(p1.y+p2.y)/2,p1.size+p2.size,angle));

  CvPoint point1;
  CvPoint point2;
  for (int i=0; i<3; i++) {
    point1 = cvPoint(three_points[i].x, three_points[i].y);
    point2 = cvPoint(three_points[(i+1)%three_points.size()].x, three_points[(i+1)%three_points.size()].y);
    cvLine(_img, point1, point2, cvScalar(0,0,255), 1);
  }
  cvLine(_img, cvPoint(p1.x,p1.y), cvPoint(p2.x,p2.y), cvScalar(0,255,0), 1);

  imgs[_offset] = _img;
}

bool TriangleTracker::insert_if_bigger (std::vector<TapirObject>& objects, TapirObject& object)
{
  if (objects.size() < 3) {
    objects.push_back(object);
    return true;
  }

  ///< \todo Finds any object that is smaller and overwrites it; should replace smallest, right?
  bool replaced = false;
  TapirObject tmpobject;
  for (int i=0; i<3; i++) {
    if (object.size > objects[i].size) {
      tmpobject = objects[i];
      objects[i] = object;
      object = tmpobject;
      replaced = true;
    }
  }
  return replaced;
}

int TriangleTracker::point_qdist (const TapirObject& p1, const TapirObject& p2)
{
  return (p2.x-p1.x)*(p2.x-p1.x)+(p2.y-p1.y)*(p2.y-p1.y);
}

double TriangleTracker::calculate_angle(const TapirObject& p1, const TapirObject& p2)
{
  return atan2(p2.y-p1.y,p2.x-p1.x);
}

REGISTER(Tracker, TriangleTracker, "Detects an object given by three markers arranged triangularly.");
