#ifndef _KHEPERA_TRACKER_
#define _KHEPERA_TRACKER_

#include "tracker.h"

using namespace Tapir;

/** Detects an object using three markings arranged in a triangle.
  * The front of the object should be marked with the point of the triangle
  * having the narrowest angle.
  *
  * @ingroup TRACKER
  * @author Xavier Bouthillier */
class TriangleTracker : public Tracker
{
public:
  TriangleTracker ();
  ~TriangleTracker () {};
  void select (IplImage** imgs, std::vector<TapirObject>& objects);
  bool init (const int width, const int height, const int imgs, unsigned int& add, const char* fname, const char* chapter);
  void deinit ();

protected:
  IplImage *_img;
  int _offset;

  /** Replace the smallest item of the list if object it bigger than it and
    * the list already contains 3 objects.
    * Otherwise it appends the object to the list
    * \param objects list of three objects
    * \param object candidate object for adding
    * \return true if object was added, false otherwise */
  bool insert_if_bigger (std::vector<TapirObject>& objects, TapirObject& object);

  int point_qdist (const TapirObject& p1, const TapirObject& p2);
  double calculate_angle(const TapirObject& p1, const TapirObject& p2);
};

#endif

